#include <iostream>
#include <string>
#include <cmath>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include "CImg-1.3.0/CImg.h"
#include "MatriceImage.h"
extern "C" {
#include "CSparse/Include/cs.h"
};


using namespace cimg_library;
using namespace std;

MatriceImage::MatriceImage(CImg<double> im){
  // Constructor : from a CImg image
  
  double* c;
  
  sizex = im.dimx();
  sizey = im.dimy();
  matR = new double[sizey*sizex];
    matG = new double[sizey*sizex];
    matB = new double[sizey*sizex];
    
    c = (double*)im.ptr();
    
    for (int i = 0; i < sizex*sizey; i++){
      matR[i] = *c;
      c++;
    }
    for (int i = 0; i < sizex*sizey; i++){
      matG[i] = *c;
      c++;
    }
    for (int i = 0; i < sizex*sizey; i++){
      matB[i] = *c;
      c++;
    }
    
}  

  
CImg<unsigned char> MatriceImage::creerIm(){
  // Create Image from this;
  double* cpar = new double[3*sizex*sizey];
  const double * c = (const double*)cpar;
  
  
  for (int i = 0; i<sizey*sizex; i++){
    *cpar = matR[i];
    cpar++;
  }
  
  for (int i = 0; i<sizey*sizex; i++){
    *cpar = matG[i];
    cpar++;
  }
  
  for (int i = 0; i<sizey*sizex; i++){
    *cpar = matB[i];
    cpar++;
  }
  
  
  CImg<double> aux = CImg<double>((const double *)c,sizex,sizey,1,3,true);
  return aux;
}



void MatriceImage::computeWLS ( double alpha, double lambda){
  // Compute the WLS filter
  double** L;
  double epsilon = 0.00001;
  int Lign = sizey, C=sizex;
  
  L = new double*[sizey];
  for (int j = 0; j<sizey; j++){
    L[j] = new double[sizex];
  }


  for (int i = 0; i<sizey; i++){
    for (int j = 0; j<sizex; j++){
      L[i][j] = log(0.299*matR[i*sizex+j] + 0.587*matG[i*sizex+j] + 0.114*matB[i*sizex+j]+epsilon);
    }
  }
  
  double A1, A2, A3, A4;
  int taille = C * Lign;
  cs_sparse A;
  A.nzmax = 5*taille;//6+8*(C-1)+(taille-2*C)*5;
  A.m = taille;
  A.n = taille;
  A.i = new int[A.nzmax];
  A.p = new int[A.n+1];
  A.x = new double[A.nzmax];
  A.nz = -1;
  
  int cpt = 0;
  
  
  for (int k=0; k<Lign*C; k++){
    int i=k/C;
    int j=k%C;
    
    A.p[k]=cpt;
    
    if(i<Lign-1){
      A1 = -lambda /(pow((abs(L[i][j]-L[i+1][j])),alpha)+epsilon);
      A.i[cpt] = k+C;
      A.x[cpt] = A1;
      cpt++;
    }else{A1=0;}
    
    if(i>0){
      A2 = -lambda /(pow((abs(L[i][j]-L[i-1][j])),alpha)+epsilon);
      A.i[cpt] = k-C;
      A.x[cpt] = A2;
      cpt++;
    }else{A2=0;}
    
    if(j<C-1){
      A3 = -lambda /(pow((abs(L[i][j]-L[i][j+1])),alpha)+epsilon);
      A.i[cpt] = k+1;
      A.x[cpt] = A3;
      cpt++;
    }else{A3=0;}
    
    if(j>0){
      A4 = -lambda /(pow((abs(L[i][j]-L[i][j-1])),alpha)+epsilon);
      A.i[cpt] = k-1;
      A.x[cpt] = A4;
      cpt++;
    }else{A4=0;}
    
    A.i[cpt] = k;
    A.x[cpt] = 1-A1-A2-A3-A4;
    cpt++;
  }
  
  A.p[taille] = cpt;
  
  int val;
  int order = 2;

  val = cs_cholsol(order,&A,matR);
  val = cs_cholsol(order,&A,matG);
  val = cs_cholsol(order,&A,matB);

  for (int j = 0; j<sizey; j++){
    delete[] L[j];
  }
  delete[] L;

  //  free(&A);
  //  delete &A;
}


MatriceImage::~MatriceImage(){
  delete[] matR;
  delete[] matG;
  delete[] matB;
}

/*
void MatriceImage::get_log(double eps){
  for (int i = 0; i<sizex*sizey; i++){
    matR[i] = log(matR[i]+eps);
    matG[i] = log(matG[i]+eps);
    matB[i] = log(matB[i]+eps);
  }
}

void MatriceImage::get_exp(double eps){
  for (int i = 0; i<sizex*sizey; i++){
    matR[i] = exp(matR[i])-eps;
    matG[i] = exp(matG[i])-eps;
    matB[i] = exp(matB[i])-eps;
  }
}
*/
